*     JLdL 26May06.
*     Copyright (C) 1999-2006 by Jorge L. deLyra.
*     Mail: <delyra@latt.if.usp.br>. Web: "http://latt.if.usp.br".
*     This program may be copied and/or distributed freely. See the
*     _ terms and conditions in the files in the doc/ subdirectory.
*     
*     Calculate the configuration-space propagator of the free theory
*     _ in four dimensions, on odd periodical lattices, for an
*     _ arbitrary value of the mass.
*     
      SUBROUTINE PROP_D4 (amass)
*     
*     Double precision everything.
      implicit double precision (a-h,o-z)
*     
*     Define the lattice size.
      parameter (n=25,nh=n/2,ns=nh**2)
*     
*     The correlation function and the periodic functions.
      dimension fcorr(-nh:nh,-nh:nh,-nh:nh,-nh:nh)
     *     ,ssin(-nh:nh),fcos(-ns:ns)
*     
*     Define some fixed parameters.
      pi=dacos(-1.0d0)
      pin=pi/dble(n)
      tpin=2.0d0*pin
*     
*     Check whether the mass is OK and calculate alpha.
      if (amass.eq.0.0d0) then
         stop 'prop_d4: ERROR: cannot calculate for exactly zero mass'
      end if
      alpha=(amass/dble(n))**2
*     
*     Calculate all the possible values of the periodic functions.
      do is=-nh,nh
         ssin(is)=dsin(pin*dble(is))**2
      end do
      do is=-ns,ns
         fcos(is)=dcos(tpin*dble(is))
      end do
*     
*     Loop over the positions.
      i4=0
      i3=0
      i2=0
      do i1=-nh,nh
         fcorr(i1,i2,i3,i4)=0.0d0
*     
*     Loop over the momentum modes.
         do k4=-nh,nh
            do k3=-nh,nh
               do k2=-nh,nh
                  do k1=-nh,nh
                     rhosq=ssin(k1)
     *                    +ssin(k2)
     *                    +ssin(k3)
     *                    +ssin(k4)
                     jfarg=k1*i1
     *                    +k2*i2
     *                    +k3*i3
     *                    +k4*i4
                     fcorr(i1,i2,i3,i4)=fcorr(i1,i2,i3,i4)
     *                    +(1.0d0/(alpha+4.0d0*rhosq))
     *                    *fcos(jfarg)
                  end do
               end do
            end do
         end do
      end do
*     
*     Normalize and print out.
      fnorm=1.0d0/fcorr(0,0,0,0)
      do i1=-nh,nh
         fcorr(i1,0,0,0)=fnorm*fcorr(i1,0,0,0)
         write(*,*) i1,fcorr(i1,0,0,0)
      end do
*     
      return
*     
      END
